/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2017 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.genomeview.gui.menu.help;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.JOptionPane;

import net.sf.genomeview.core.Configuration;
import net.sf.genomeview.core.Icons;
import net.sf.genomeview.data.Model;
import net.sf.genomeview.gui.MessageManager;


/**
 * Action to show the about dialog.
 * 
 * @author Thomas Abeel
 * @author thpar
 * 
 */
public class ShowAboutDialogAction extends AbstractAction {

    private Model model;

    private static final long serialVersionUID = 4182067300462615334L;

    public ShowAboutDialogAction(Model model) {
        super(MessageManager.getString("helpmenu.about"));
        this.model = model;
    }

    @Override
    public void actionPerformed(ActionEvent e) {

        String message = MessageManager.formatMessage("helpmenu.message", new Object[]{Configuration.version(),Configuration.getDirectory().toString()});
        
        JOptionPane.showMessageDialog(model.getGUIManager().getParent(), message, MessageManager.formatMessage("helpmenu.version", new Object[]{Configuration.version()}),
                JOptionPane.INFORMATION_MESSAGE,Icons.MINILOGO_ICON);
    }

}
