/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2017 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.jannot.alignment.maf;

import net.sf.jannot.Located;
import net.sf.jannot.Location;
import net.sf.jannot.Strand;
import net.sf.jannot.refseq.Sequence;

/**
 * @author Thomas Abeel
 * 
 */
public abstract class AbstractAlignmentBlock implements Comparable<AbstractAlignmentBlock>, Located,
		Iterable<AbstractAlignmentSequence> {
	/* Buffers for finding nucleotides */
	// protected int[] position = null;
	// private Entry ref = null;
	private Location loc = new Location(0, 0);

	public AbstractAlignmentBlock(int start, int end) {
		this.loc = new Location(start, end);
	}

	// public void setRef(Entry entry) {
	// this.ref = entry;
	// update(entry);
	// }

	// /**
	// * @param reference
	// */
	// private void update(Entry reference) {
	// this.ref = reference;
	//
	// for (AlignmentSequence as : this) {
	// System.out.println(as);
	// if (as.entry() == ref) {
	//
	// return;
	// }
	// }
	//
	// }

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(AbstractAlignmentBlock o) {
		int thisVal = this.hashCode();
		int anotherVal = o.hashCode();
		return (thisVal < anotherVal ? -1 : (thisVal == anotherVal ? 0 : 1));
	}

	//
	// /*
	// * (non-Javadoc)
	// *
	// * @see net.sf.jannot.Located#getLocation()
	// */
	// @Override
	// public Location getLocation() {
	// return loc;
	// }
	@Override
	public int start() {
		return loc.start;
	}

	@Override
	public int end() {
		return loc.end;
	}

	public int length() {
		return loc.length();
	}

	/**
	 * @param as
	 * @return
	 */
	public abstract void add(AbstractAlignmentSequence as);

	/**
	 * @param i
	 * @return
	 */
	public abstract AbstractAlignmentSequence getAlignmentSequence(int i);

	/**
	 * @return
	 */
	public abstract int size();

}
