/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2017 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.jannot.alignment.maf;

import net.sf.jannot.Strand;
import net.sf.jannot.refseq.Sequence;

/**
 * @author Thomas Abeel
 * 
 */
public class LazyAlignmentSequence extends AbstractAlignmentSequence {

	private LazyAlignmentBlock parent;

	/**
	 * @param selectedChrom
	 * @param nucStart
	 * @param alignmentLength
	 * @param strand2
	 */
	public LazyAlignmentSequence(String id, int nucStart, int alignmentLength, Strand strand, LazyAlignmentBlock parent) {
		super(id, nucStart, alignmentLength, strand);
		this.parent = parent;
	}

	/* This field needs to be set by the lazy load method of the parent */
	private Sequence seq = null;

	@Override
	public Sequence seq() {
		if (seq == null) {
			parent.lazyLoad();

		}

		return seq;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sf.jannot.alignment.maf.AbstractAlignmentSequence#end()
	 */
	@Override
	public int end() {
		if (seq == null) {
			parent.lazyLoad();
		}
		return super.end();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sf.jannot.alignment.maf.AbstractAlignmentSequence#start()
	 */
	@Override
	public int start() {
		if (seq == null)
			parent.lazyLoad();
		return super.start();
	}

	/**
	 * @param seq2
	 */
	public void setSeq(Sequence seq) {
		this.seq = seq;

	}

}
