/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2017 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.jannot.parser;

import java.io.InputStream;

import net.sf.jannot.DataKey;
import net.sf.jannot.Entry;
import net.sf.jannot.EntrySet;
import net.sf.jannot.StringKey;
import net.sf.jannot.wiggle.FloatArrayWiggle;
import be.abeel.io.LineIterator;
import cern.colt.list.FloatArrayList;

public class BedGraphParser extends Parser {

	/**
	 * @param dataKey
	 */
	public BedGraphParser(DataKey datakey) {
		super(datakey);

	}

	@Override
	public EntrySet parse(InputStream is, EntrySet set) {
		if (set == null)
			set = new EntrySet();
		LineIterator it = new LineIterator(is);
		it.setSkipComments(true);
		it.setCommentIdentifier("#");
		it.addCommentIdentifier("browser");
		it.addCommentIdentifier("track");

		FloatArrayList values = new FloatArrayList();
		String last = "";
		Entry e = null;
		for (String line : it) {
			String[] arr = line.split("\t");
			int start = Integer.parseInt(arr[1]);
			int end = Integer.parseInt(arr[2]);
			if (!last.equals(arr[0])) {
				last = arr[0];
				
				if (e != null){
					e.add(dataKey, new FloatArrayWiggle(values.elements()));
					System.out.println("Adding: "+e+"\t"+values.size());
					values=new FloatArrayList();
				}
				e = set.getOrCreateEntry(arr[0]);
			}
			float val = Float.parseFloat(arr[3]);
			/* Make sure the array is big enough */
			if(end>values.size())
				values.setSize(end);
			for (int i = start; i < end; i++) {
				values.set(i, val);
			}

		}
		
		return set;
	}


}
