/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2017 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.jannot.refseq;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;

import be.abeel.io.LineIterator;
/**
 * 
 * @author Thomas Abeel
 *
 */
public class FaidxIndex {

	
	class IndexEntry{
		long len,start,lineLen,byteLen;

		/**
		 * @param len
		 * @param start
		 * @param lineLen
		 * @param byteLen
		 */
		IndexEntry(long len, long start, long lineLen, long byteLen) {
			super();
			this.len = len;
			this.start = start;
			this.lineLen = lineLen;
			this.byteLen = byteLen;
		}
	}
	private HashMap<String,IndexEntry>mapping=new HashMap<String, IndexEntry>();
	private String[]names;


	public FaidxIndex(InputStream is) {
		LineIterator it = new LineIterator(is);
		ArrayList<String>lines=new ArrayList<String>();
		for (String line : it) {
			lines.add(line);
		}
		names=new String[lines.size()];
		
		int idx=0;
		for(String line:lines){
			String[] arr = line.split("\t");
			try {
				names[idx++]=arr[0];
				IndexEntry ie=new IndexEntry(Long.parseLong(arr[1]), Long.parseLong(arr[2]), Long.parseLong(arr[3]), Long.parseLong(arr[4]));
				mapping.put(arr[0],ie);

				
			} catch (ArrayIndexOutOfBoundsException e) {
				System.err.println("Could not parse: " + line);
				while(it.hasNext()){
					System.err.println(it.next());
				}
			}
		}

	}

	public String[] names() {
		return names;
	}
	
	public IndexEntry get(String name){
		return mapping.get(name);
	}

}
