/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2017 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.jannot.source;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import net.sf.jannot.EntrySet;
import net.sf.jannot.exception.ReadFailedException;

/**
 * 
 * @author Thomas Abeel
 * 
 */
public abstract class DataSource implements Comparable<DataSource> {

	protected Locator locator=null;
	
	protected DataSource(Locator l){
		this.locator=l;
	}
	
	@Override
	public int compareTo(DataSource o) {
		return this.toString().compareTo(o.toString());
	}

	public EntrySet read() throws ReadFailedException {
		return read(null);
	}

	public abstract EntrySet read(EntrySet add) throws ReadFailedException;

	public abstract boolean isIndexed();

	public abstract long size();

	protected static long size(URL url, File file) {
		long size = 0;
		if (url != null)
			try {
				size = url.openConnection().getContentLength();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		if (file != null)
			size = file.length();

		return size;

	}

	public Locator getLocator(){
		return locator;
	}
}
