/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2017 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.jannot.source.cache;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URL;
import java.util.HashSet;
import java.util.Set;

import atk.util.MD5Tools;
import be.abeel.io.ExtensionFileFilter;
import net.sf.jannot.source.DataSource;
import net.sf.jannot.source.FileSource;


public class SourceCache {

	public static File cacheDir=new File(System.getProperty("user.home"));;

	public static boolean contains(URL url) {
		
		
		System.out.println("URL cache: "+cacheDir);
		if(!cacheDir.exists())
			cacheDir.mkdir();
		File[]files=cacheDir.listFiles(new ExtensionFileFilter("url"));
		Set<String>names=new HashSet<String>();
		String md5=MD5Tools.md5(url.toString());
		for(File file:files){
			names.add(file.getName());
		
		}
		return names.contains(md5+".url");
		
	}

	public static DataSource get(URL url) throws IOException {
		System.out.println("Retrieving from cache: "+url);
		return  new FileSource(new File(cacheDir,MD5Tools.md5(url.toString())+".url"));
	}

	public static OutputStream startCaching(URL url) throws FileNotFoundException {
		return  new FileOutputStream(new File(cacheDir,MD5Tools.md5(url.toString())+".tmp"));
	}

	/**
	 * This method should only be called when the data is completely written.
	 * 
	 * @param url
	 */
	public static void finish(URL url) {
		File f=new File(cacheDir,MD5Tools.md5(url.toString())+".tmp");
		f.renameTo(new File(cacheDir,MD5Tools.md5(url.toString())+".url"));
		
	}

}
